@@ -616,7 +616,7 @@ class LMContext(BaseContext):
             vac_count = self.get_item_count_by_id(8148)
             vac_speed = min(self.get_item_count_by_id(8064), 5)
 
-            if not self.trap_link.check_vac_trap_active():
+            if not self.trap_link.check_vac_trap_active() and vac_count > 0:
                 for item in [8064, 8148]:
                     lm_item_name = self.item_names.lookup_in_game(item)
                     lm_item = ALL_ITEMS_TABLE[lm_item_name]
