@@ -1,11 +1,11 @@
-import json
-import os, yaml
+import json, os
+from random import Random
+
 import Utils
 
 from CommonClient import logger
 
 from gclib.gcm import GCM
-from gclib.dol import DOL
 from gclib.rarc import RARC
 from gclib.yaz0_yay0 import Yay0
 
@@ -41,15 +41,17 @@ class LuigisMansionRandomizer:
         # After verifying, this will also read the entire iso, including system files and their content
         self.gcm = GCM(self.clean_iso_path)
         self.gcm.read_entire_disc()
-        self.dol = DOL()
+
+        # Set the random's seed for uses in other files.
+        self.random = Random()
+        local_seed: str = str(self.output_data["Seed"])
+        self.random.seed(local_seed)
 
         # Change game ID so save files are different
         logger.info("Updating the ISO game id with the AP generated seed.")
-        self.seed = self.output_data["Seed"]
-        magic_seed = str(self.seed)
         bin_data = self.gcm.read_file_data("sys/boot.bin")
         bin_data.seek(0x01)
-        bin_data.write(sbf.string_to_bytes(magic_seed, len(magic_seed)))
+        bin_data.write(sbf.string_to_bytes(local_seed, len(local_seed)))
         self.gcm.changed_files["sys/boot.bin"] = bin_data
 
         # Updates the Game USA folder to have the correct ghost file we expect.
@@ -139,8 +141,8 @@ class LuigisMansionRandomizer:
         update_character_info(self.jmp_character_info_table, self.output_data)
         update_item_info_table(self.jmp_item_info_table, self.output_data)
         update_item_appear_table(self.jmp_item_appear_table, self.output_data)
-        update_treasure_table(self.jmp_treasure_table, self.jmp_character_info_table, self.output_data)
-        update_treasure_table(self.jmp_treasure_table, self.jmp_teiden_character_info_table, self.output_data)
+        update_treasure_table(self, self.jmp_treasure_table, self.jmp_character_info_table, self.output_data)
+        update_treasure_table(self, self.jmp_treasure_table, self.jmp_teiden_character_info_table, self.output_data)
         update_furniture_info(self.jmp_furniture_info_table, self.jmp_item_appear_table, self.output_data)
         update_event_info(self.jmp_event_info_table, bool_boo_checks, self.output_data)
 
@@ -149,17 +151,17 @@ class LuigisMansionRandomizer:
         update_key_info(self.jmp_key_info_table, self.output_data)
         update_obj_info(self.jmp_obj_info_table)
         update_generator_info(self.jmp_generator_info_table)
-        update_enemy_info(self.jmp_enemy_info_table, self.output_data)
+        update_enemy_info(self, self.jmp_enemy_info_table, self.output_data)
         update_teiden_observer_info(self.jmp_observer_info_table,
             self.jmp_teiden_observer_info_table, bool_speedy_spirits)
         if bool_speedy_spirits:
             update_teiden_enemy_info(self.jmp_enemy_info_table, self.jmp_teiden_enemy_info_table)
 
         logger.info("Updating Boos, other iyapoos, and rooms/events...")
-        update_boo_table(self.jmp_boo_table, self.output_data)
+        update_boo_table(self, self.jmp_boo_table, self.output_data)
         update_iyapoo_table(self.jmp_iyapoo_table, self.output_data)
         if int_spookiness != 0:
-            update_room_info(self.jmp_room_info_table, int_spookiness)
+            update_room_info(self, self.jmp_room_info_table, int_spookiness)
         update_event_info(self.jmp_map3_event_info_table, bool_boo_checks, self.output_data)
 
         # Updates all the data entries in each jmp table in the szp file.
@@ -197,13 +199,6 @@ class LuigisMansionRandomizer:
         bool_randomize_mice: bool = bool(self.output_data["Options"]["gold_mice"])
         bool_hidden_mansion: bool = bool(self.output_data["Options"]["hidden_mansion"])
         bool_start_boo_radar: bool = not bool(self.output_data["Options"]["boo_radar"])
-        walk_speed: int = int(self.output_data["Options"]["walk_speed"])
-        bool_fear_anim_enabled: bool =  bool(self.output_data["Options"]["enable_fear_animation"])
-        bool_pickup_anim_enabled: bool = bool(self.output_data["Options"]["enable_pickup_animation"])
-        player_name: str = str(self.output_data["Name"])
-        king_boo_health: int = int(self.output_data["Options"]["king_boo_health"])
-        random_spawn: str = str(self.output_data["Options"]["spawn"])
-        door_model_rando_on: bool = bool(self.output_data["Options"]["door_model_rando"])
 
         # Boo related options
         bool_boo_checks: bool = True if self.output_data["Options"]["boo_gates"] == 1 else False
@@ -218,15 +213,13 @@ class LuigisMansionRandomizer:
         bool_portrait_hints: bool = True if self.output_data["Options"]["portrait_hints"] == 1 else False
 
         logger.info("Updating all the main.dol offsets with their appropriate values.")
-        self.gcm, self.dol = update_dol_offsets(self.gcm, self.dol, self.seed,
-            start_inv_list, walk_speed, player_name, random_spawn, king_boo_health, bool_fear_anim_enabled,
-            bool_pickup_anim_enabled, bool_boo_rando_enabled, door_model_rando_on)
+        update_dol_offsets(self)
 
         logger.info("Updating all of the common events with the customized version.")
-        self.gcm = update_common_events(self.gcm, bool_randomize_mice, bool_start_vacuum)
+        update_common_events(self, bool_randomize_mice, bool_start_vacuum)
 
         logger.info("Updating the intro and lab events with the customized version.")
-        self.gcm = update_intro_and_lab_events(self.gcm, bool_hidden_mansion, max_health, start_inv_list, bool_start_boo_radar,
+        update_intro_and_lab_events(self, bool_hidden_mansion, max_health, start_inv_list, bool_start_boo_radar,
             door_to_close_list, bool_start_vacuum)
 
         if bool_boo_checks:
@@ -245,28 +238,28 @@ class LuigisMansionRandomizer:
                     self.jmp_event_info_table.info_file_field_entries = list(filter(lambda info_entry: not (
                         info_entry["EventNo"] == int(event_no)), self.jmp_event_info_table.info_file_field_entries))
                     continue
-                self.gcm = update_boo_gates(self.gcm, event_no, required_boo_count,
+                update_boo_gates(self, event_no, required_boo_count,
                     bool_boo_rando_enabled, str_move_type)
 
         logger.info("Updating the blackout event with the customized version.")
-        self.gcm = update_blackout_event(self.gcm)
+        update_blackout_event(self)
 
         logger.info("Updating Clairvoya's event with the customized version.")
-        self.gcm = randomize_clairvoya(self.gcm, req_mario_count, hint_dist, madam_hint_dict, self.seed)
+        randomize_clairvoya(self, req_mario_count, hint_dist, madam_hint_dict)
 
         logger.info("Updating common events with the generated in-game hints.")
-        self.gcm = write_in_game_hints(self.gcm, hint_dist, hint_list, max_health, self.seed)
+        write_in_game_hints(self, hint_dist, hint_list, max_health)
 
         logger.info("Updating the spawn event...")
-        self.gcm = update_spawn_events(self.gcm)
+        update_spawn_events(self)
 
         if bool_portrait_hints:
             logger.info("Portrait Hints are enabled, updating portrait ghost hearts with the generated in-game hints.")
-            self.gcm = write_portrait_hints(self.gcm, hint_dist, hint_list, self.seed)
+            write_portrait_hints(self, hint_dist, hint_list)
 
         if bool_randomize_music:
             logger.info("Randomized Music is enabled, updating all events with various in-game music.")
-            self.gcm = randomize_music(self.gcm, self.seed)
+            randomize_music(self)
 
         self.update_map_jmp_tables()
 
