@@ -30,6 +30,7 @@ all_random_settings = {
     LMOptions.Lightsanity.internal_name:               "random",
     LMOptions.Walksanity.internal_name:                "random",
     LMOptions.SpeedySpirits.internal_name:             "random",
+    LMOptions.Grassanity.internal_name:                "random",
     LMOptions.BooGates.internal_name:                  "random",
     LMOptions.KingBooHealth.internal_name:             "random",
     LMOptions.BoolossusDifficulty.internal_name:       "random",
@@ -83,6 +84,7 @@ allsanity_settings = {
     LMOptions.Lightsanity.internal_name:               "true",
     LMOptions.Walksanity.internal_name:                "true",
     LMOptions.SpeedySpirits.internal_name:             "true",
+    LMOptions.Grassanity.internal_name:                "true",
 }
 
 money_settings = {
