@@ -1,8 +1,7 @@
-from . import LMWorld, LMWeb
-from .LMClient import launch
+from . import GenericWorld, GenericWeb
 
 """
-Luigi's Mansion is an adventure game starring everyone's favorite plumber brother, Luigi. World Registration
+Generic World Registration
 
 This file contains the metadata and class references for the luigismansion world.
 """
@@ -18,6 +17,6 @@ AUTHOR = author
 VERSION = version
 
 # Plugin entry points
-WORLD_CLASS = LMWorld
-WEB_WORLD_CLASS = LMWeb
-CLIENT_FUNCTION = launch
+WORLD_CLASS = GenericWorld
+WEB_WORLD_CLASS = GenericWeb
+CLIENT_FUNCTION = None
