@@ -9,6 +9,7 @@ class LinkBase(ABC):
     friendly_name: str
     slot_name: str
     network_engine: ArchipelagoNetworkEngine
+    enable_logger: bool = True
 
     def __init__(self, friendly_name, slot_name, network_engine: ArchipelagoNetworkEngine):
         self.friendly_name = friendly_name
@@ -18,3 +19,12 @@ class LinkBase(ABC):
     def is_enabled(self) -> bool:
         """ Determines if the given link is enabled in the client. """
         return self.friendly_name in self.network_engine.get_tags()
+
+    def set_logs(self, value: bool):
+        """
+        Sets the Link's client logger to the given value.
+        
+        :param value: If True, we enable client messages.
+            Otherwise no link related messages will be sent to the client.
+        """
+        self.enable_logger = value
